%% clear
clc;
clear;
close all;
format compact;
format short g;

%% Set up FEM struct and pre-assemble
fem = getInitFem;
prob.fem.dof = getDof(fem);

prob.theta(1).fun = @(mu) mu(1);
prob.theta(2).fun = @(mu) mu(2);

prob = performPreAssembling(fem,prob);

%% Compute matrices for discrete inner porduct
[fem.const{2:2:end}] = deal(1);
[C,D] = assemble(fem,'Out',{'C','D'});

L2 = D;
L2 = (L2+L2')/2;

H1 = C+D;
H1 = (H1+H1')/2;

% ==== TBD ====
% En = ....

prob.fem.innProd.L2 = L2;
prob.fem.innProd.H1 = H1;
prob.fem.innProd.En = En;

%% Aufgabe 1: Compare coercivity constants for different inner products

% compute coercivities and inf-sup-constants
N = 40;
alpha = zeros(2,N);
mu1_list = logspace(-2,2,N);
mu2 = 1;

for n=1:numel(mu1_list)
  mu = [mu1_list(n);mu2];
  
  alpha(1,n) = getCoercivity(prob,mu, 'H1');
  alpha(2,n) = getCoercivity(prob,mu, 'En');

end

figure(1);
loglog(mu1_list',alpha', '-x');
legend('H1-Norm', 'En-Norm');
xlabel \mu_1;
ylabel \alpha(\mu_1);


%% Set up POD

% parameter space w.r.t. \mu_1
D = [.01 100];
logD = log10(D);

% number of snapshots n_train
n_train = 101;

% parameter training set
Xi_train = [logspace(logD(1),logD(2),n_train); ...
            ones(1,n_train)];

% compute the POD basis
prob.POD = computePODBasis(prob,Xi_train,'H1');


%% Aufgabe 3: Compare error and error estimator
n_test = 40;
Xi_test = [10.^(sort(rand(1,n_test)*diff(logD)+logD(1))); ...
           ones(1,n_test)];

err_rb_POD = zeros(n_test,1);
errEst_POD = NaN*ones(n_test,1);

innProd = prob.fem.innProd.(prob.POD.Norm);
for n=1:n_test
  mu = Xi_test(:,n);
  
  u = getSol(prob,mu);
  [dummy,u_rb_POD] = getPODAppr(prob,mu);
  
  e = u-u_rb_POD;
  err_rb_POD(n) = sqrt(e'*innProd*e);
  
  errEst_POD(n) = getErrEst(prob,mu,u_rb_POD,'H1');
end

figure(2);
e = [err_rb_POD errEst_POD];
loglog(Xi_test(1,:),e,'x-')
title('Error and error estimator')
legend('Error', 'Error Bound');
xlabel \mu_1

figure(3);
semilogx(Xi_test(1,:),errEst_POD./err_rb_POD,'x-');
title('Effectivity')
xlabel \mu_1

